<?php
/*
Plugin Name: WP Freeton
Plugin URI: https://github.com/weborsha/WpFreeTON
Description: Login with FreeTon Wallet
Version: 1.0.0
Author: FreeTonTask
Author URI: https://github.com/weborsha/WpFreeTON
Text Domain: freeton-wp
Domain Path: /languages
*/


class webwpfreeton
{
    public $textdomain = 'freeton-wp';

    function __construct()
    {
        add_action('init', array(&$this, 'web_wpfreeton_textdomain'));
        add_action('admin_init', array(&$this, 'wpfreeton_admin_init'));
        add_action('admin_menu', array(&$this, 'admin_menu'));
        add_action('wp_enqueue_scripts', array(&$this, 'enqueue_scripts'));
        add_action('wp_ajax_web_wpfreeton_auth_customer', array(&$this, 'web_wpfreeton_auth_customer'));
        add_action('wp_ajax_web_wpfreeton_auth_customer_with_website', array(&$this, 'web_wpfreeton_auth_customer_with_website'));
        add_action('wp_ajax_web_wpfreeton_activate_customer', array(&$this, 'web_wpfreeton_activate_customer'));
        add_action('wp_ajax_wpfreeton_ajax_login', array(&$this, 'wpfreeton_ajax_login'));
        add_action('wp_ajax_wpfreeton_update_password_action', array(&$this, 'wpfreeton_update_password_action'));
        add_action('wp_ajax_wpfreeton_enter_password_action', array(&$this, 'wpfreeton_enter_password_action'));
        add_action('wp_ajax_wpfreeton_ajax_login_with_email', array(&$this, 'wpfreeton_ajax_login_with_email'));
        add_action('wp_ajax_wpfreeton_ajax_register', array(&$this, 'wpfreeton_ajax_register'));
        add_action('wp_ajax_wpfreeton_forgot_password', array(&$this, 'wpfreeton_forgot_password'));
        add_action('wp_ajax_nopriv_wpfreeton_ajax_login', array(&$this, 'wpfreeton_ajax_login'));
        add_action('wp_ajax_nopriv_wpfreeton_ajax_login_with_email', array(&$this, 'wpfreeton_ajax_login_with_email'));
        add_action('wp_ajax_nopriv_wpfreeton_ajax_register', array(&$this, 'wpfreeton_ajax_register'));
        add_action('wp_ajax_nopriv_wpfreeton_update_password_action', array(&$this, 'wpfreeton_update_password_action'));
        add_action('wp_ajax_nopriv_wpfreeton_enter_password_action', array(&$this, 'wpfreeton_enter_password_action'));
        add_action('wp_ajax_nopriv_wpfreeton_forgot_password', array(&$this, 'wpfreeton_forgot_password'));
        add_action('activated_plugin', array(&$this, 'wpfreeton_activation_redirect'));
        add_action('wp_head', array(&$this, 'wpfreeton_custom_css'));

        add_action('wpfreeton_rest_api_init', array(&$this, 'wpfreeton_register_rest_route'));
        add_filter('manage_users_columns', array(&$this, 'wpfreeton_modify_user_table'));
        add_filter('manage_users_custom_column', array(&$this, 'wpfreeton_modify_user_table_row'), 10, 3);


        add_shortcode('web_wpfreeton', array(&$this, 'shortcode'));
        add_shortcode('web_wpfreeton_metas', array(&$this, 'web_wpfreeton_metas'));

    }

    function wpfreeton_activation_redirect($plugin)
    {
        if ($plugin == plugin_basename(__FILE__)) {
            exit(wp_redirect(admin_url('admin.php?page=web-freeton-wp')));
        }
    }

    function web_wpfreeton_textdomain()
    {
        $web_wpfreeton_lang_dir = dirname(plugin_basename(__FILE__)) . '/languages/';
        $web_wpfreeton_lang_dir = apply_filters('web_wpfreeton_languages_directory', $web_wpfreeton_lang_dir);

        load_plugin_textdomain($this->textdomain, false, $web_wpfreeton_lang_dir);
    }

    function wpfreeton_admin_init()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_token'])) $options['web_token'] = '';
        if (!isset($options['web_styles_status'])) $options['web_styles_status'] = '0';

        register_setting('web-freeton-wp', 'web_wpfreeton_settings', array(&$this, 'settings_validate'));
        register_setting('web-freeton-wp-styles', 'web_wpfreeton_settings', array(&$this, 'settings_validate'));

        add_settings_section('web-freeton-wp-styles', '', array(&$this, 'wpfreeton_section_intro'), 'web-freeton-wp-styles');
        add_settings_field('web_styles_status', __('Enable custom styles', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_enable_custom_style'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);

        if ($options['web_styles_status']) {
//            add_settings_field('web_styles_title1', 'tyuiuy', array(&$this, 'wpfreeton_section_title'), 'web-freeton-wp-styles');
            add_settings_field('web_styles_title', __('Primary button', 'freeton-wp'), array(&$this, 'wpfreeton_section_title'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_button_background', __('button background color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_button_background_color'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_button_border_color', __('button border color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_button_border_color'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_button_border_radius', __('button border radius', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_button_border_radius'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_button_border_width', __('button border width', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_button_border_width'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_button_text_color', __('button text color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_button_text_color'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);

//            add_settings_section('web_styles_title2', '', array(&$this, 'wpfreeton_section_title'), 'web-freeton-wp-styles');
            add_settings_field('web_styles_title2', __('Secondary button', 'freeton-wp'), array(&$this, 'wpfreeton_section_title'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);

            add_settings_field('web_styles_button_background2', __('secondary button background color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_button_background_color2'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_button_border_color2', __('secondary button border color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_button_border_color2'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_button_border_radius2', __('secondary button border radius', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_button_border_radius2'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_button_border_width2', __('secondary button border width', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_button_border_width2'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_button_text_color2', __('secondary button text color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_button_text_color2'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);


            add_settings_field('web_styles_title3', __('Inputs', 'freeton-wp'), array(&$this, 'wpfreeton_section_title'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);

            add_settings_field('web_styles_input_background', __('input background color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_button_background_color'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_input_border_color', __('input border color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_input_border_color'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_input_border_radius', __('input border radius', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_input_border_radius'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_input_border_width', __('input border width', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_input_border_width'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_input_text_color', __('input text color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_input_text_color'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_input_placeholder_color', __('input placeholder color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_input_placeholder_color'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);

            add_settings_field('web_styles_title4', __('Box', 'freeton-wp'), array(&$this, 'wpfreeton_section_title'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_box_background_color', __('box background color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_box_background_color'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);


            add_settings_field('web_styles_title5', __('Labels', 'freeton-wp'), array(&$this, 'wpfreeton_section_title'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_labels_text_color', __('label text color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_labels_text_color'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_labels_font_size', __('label font size', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_labels_font_size'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);



            add_settings_field('web_styles_title6', __('Titles', 'freeton-wp'), array(&$this, 'wpfreeton_section_title'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_title_color', __('title color', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_title_color'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
            add_settings_field('web_styles_title_font_size', __('title font size', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_style_title_font_size'), 'web-freeton-wp-styles', 'web-freeton-wp-styles', ['label_for' => '', 'class' => 'iwpfreetonlabel']);

        }

        add_settings_section('web-freeton-wp', '', array(&$this, 'wpfreeton_section_intro'), 'web-freeton-wp');


        $ghgfd = '';
        if ($options['web_token']) {
            $ghgfd = ' none';
        }
	    add_settings_field('wpfreeton_debot_wallet', __('Enter Debot address', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_debot_wallet'), 'web-freeton-wp', 'web-freeton-wp', ['label_for' => '', 'class' => 'iwpfreetonlabel']);

	    add_settings_field('web_email_login', __('Enable email login', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_email_login'), 'web-freeton-wp', 'web-freeton-wp', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
        add_settings_field('web_password_login', __('Enable password login', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_password_login'), 'web-freeton-wp', 'web-freeton-wp', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
        add_settings_field('wpfreeton_redirect_url', __('Enter redirect url', 'freeton-wp'), array(&$this, 'wpfreeton_setting_web_url_redirect'), 'web-freeton-wp', 'web-freeton-wp', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
        add_settings_field('web_position_form', __('Enable fix position', 'freeton-wp'), array(&$this, 'web_position_form'), 'web-freeton-wp', 'web-freeton-wp', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
        add_settings_field('web_use_freeton_wallet_for_username', __('Use TON wallet for username', 'freeton-wp'), array(&$this, 'web_use_freeton_wallet_for_username'), 'web-freeton-wp', 'web-freeton-wp', ['label_for' => '', 'class' => 'iwpfreetonlabel']);
        add_settings_field('web_default_username', __('Default username', 'freeton-wp'), array(&$this, 'setting_default_username'), 'web-freeton-wp', 'web-freeton-wp', ['label_for' => '', 'class' => 'iwpfreetonlabel related_to_upnfu']);
        add_settings_field('web_default_nickname', __('Default nickname', 'freeton-wp'), array(&$this, 'setting_default_nickname'), 'web-freeton-wp', 'web-freeton-wp', ['label_for' => '', 'class' => 'iwpfreetonlabel related_to_upnfu']);
    }

    function admin_menu()
    {

        $icon_url = 'dashicons-superhero-alt';
        $page_hook = add_menu_page(
            __('WP FreeTon setting', 'freeton-wp'),
            __('WP FreeTon setting', 'freeton-wp'),
            'manage_options',
            'web-freeton-wp',
            array(&$this, 'settings_page'),
            $icon_url
        );
        add_submenu_page('web-freeton-wp', __('style settings', 'freeton-wp'), __('Style Settings', 'freeton-wp'), 'manage_options', 'web-freeton-wp-styles', array(&$this, 'style_settings_page'));

        add_action('admin_print_styles-' . $page_hook, array(&$this, 'admin_custom_css'));
        wp_enqueue_script('web-freeton-wp-admin-select2-js', plugins_url('/scripts/select2.full.min.js', __FILE__), array('jquery'), true, true);

    }

    function admin_custom_css()
    {
        wp_enqueue_style('web-freeton-wp-admin', plugins_url('/styles/freeton-wp-admin.css', __FILE__));
        wp_enqueue_style('web-freeton-wp-admin-select2-style', plugins_url('/styles/select2.min.css', __FILE__));
    }

    function settings_page()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_freeton_wallet'])) $options['web_freeton_wallet'] = '';
        if (!isset($options['web_token'])) $options['web_token'] = '';

        ?>
        <div class="wrap">
            <div id="icon-themes" class="icon32"></div>
            <h2><?php _e('Settings', 'freeton-wp'); ?></h2>
            <?php if (isset($_GET['settings-updated']) && $_GET['settings-updated']) {

                ?>
                <div id="setting-error-settings_updated" class="updated settings-error">
                    <p><strong><?php _e('Settings saved.', 'freeton-wp'); ?></strong></p>
                </div>
            <?php } ?>
            <form action="options.php" method="post" id="iuytfrdghj">
                <?php settings_fields('web-freeton-wp'); ?>
                <?php do_settings_sections('web-freeton-wp'); ?>


                <p class="submit">
                    <input type="submit" class="button-primary"
                           value="<?php _e('Save Changes', 'freeton-wp'); ?>"/></p>
            </form>

            <script>
                <?php

                ?>
                jQuery(function ($) {
                    var web_freeton_wallet_ccodeG = '1';
                    $(window).load(function () {

                        $('.loiuyt').click();
                        $('.refreshShop').click();
                        $("#web_freeton_wallet_ccode").select2();

                        <?php
                        if (empty($options['web_token'])) {
                        ?>
                        $('.authwithwebsite').click();
                        <?php } ?>

                    });

                    var edf2 = $('#web_wpfreeton_settings_use_freeton_wallet_for_username');
                    var edf4 = $('#web_default_gateways');
                    console.clear();
                    console.log(edf4);
                    var web_body = $('body');
                    var related_to_login = $('.related_to_login');
                    var related_to_upnfu = $('.related_to_upnfu');
                    var related_to_defaultgateway = $('.related_to_defaultgateway');
                    var related_to_customgateway = $('.related_to_customgateway');
                    var related_to_zenziva = $('.related_to_zenziva');

                    if (edf.is(':checked')) {
                        console.log('is checked!');
                        related_to_login.css('display', 'table-row');
                        // $("#web_freeton_wallet_ccode").chosen();


                    } else {
                        console.log('is not checked!');

                        related_to_login.css('display', 'none');
                    }


                    if (edf2.is(':checked')) {
                        console.log('is checked!');
                        // $("#web_freeton_wallet_ccode").chosen();
                        related_to_upnfu.css('display', 'none');


                    } else {
                        console.log('is not checked!');
                        related_to_upnfu.css('display', 'table-row');

                    }

                    if (edf3.is(':checked')) {
                        console.log('is checked!');
                        // $("#web_freeton_wallet_ccode").chosen();
                        related_to_defaultgateway.css('display', 'table-row');
                        $('.rltll').css('display', 'none');


                    } else {
                        console.log('is not checked!');
                        related_to_defaultgateway.css('display', 'none');


                    }

                    if (edf4.val() == 'custom' && edf3.is(':checked')) {
                        console.log('is checked!');
                        // $("#web_freeton_wallet_ccode").chosen();
                        related_to_customgateway.css('display', 'table-row');


                    } else {
                        console.log('is not checked!');
                        related_to_customgateway.css('display', 'none');


                    }
                    if (edf4.val() == 'zenziva' && edf3.is(':checked')) {
                        console.log('is checked!');
                        // $("#web_freeton_wallet_ccode").chosen();
                        related_to_zenziva.css('display', 'table-row');


                    } else {
                        console.log('is not checked!');
                        related_to_zenziva.css('display', 'none');


                    }

                    $('#web_wpfreeton_settings_use_freeton_wallet_for_username').change(
                        function () {
                            if (this.checked && this.value == '1') {
                                console.log('change is checked!');

                                // $("#web_freeton_wallet_ccode").chosen();
                                related_to_upnfu.css('display', 'none');

                            } else {
                                console.log('change is not checked!');
                                related_to_upnfu.css('display', 'table-row');

                            }
                        });

                    web_body.on('click', '.auth',
                        function () {
                            var wpfreeton_freeton_wallet = $('#wpfreeton_freeton_wallet').val();
                            var web_freeton_wallet_ccode = $('#web_freeton_wallet_ccode').val();
                            web_freeton_wallet_ccodeG = web_freeton_wallet_ccode;
                            // alert(web_freeton_wallet_ccode);
                            // return;
                            if (wpfreeton_freeton_wallet) {
                                $('.wpfreeton_freeton_wallet_label th').html('enter code messaged to you!');
                                $('#wpfreeton_freeton_wallet').css('display', 'none');
                                $('#wpfreeton_secod').css('display', 'inherit');
                                $('.i34').css('display', 'inline-block');
                                $('.i35').css('display', 'none');
                                $('.web_freeton_wallet_ccode_wrap').css('display', 'none');
                                wpfreeton_freeton_wallet = web_freeton_wallet_ccode + wpfreeton_freeton_wallet;
                                $.ajax({
                                    type: "GET",
                                    url: ajaxurl,
                                    data: {
                                        action: 'web_wpfreeton_auth_customer',
                                        freeton_wallet: wpfreeton_freeton_wallet,
                                    }
                                }).done(function (msg) {
                                    if (msg) {
                                        var arr = JSON.parse(msg);
                                        console.log(arr);
                                    }
                                    // $('form#iuytfrdghj').submit();

                                });

                            }
                        });

                    web_body.on('click', '.authwithwebsite',
                        function () {
                            var wpfreeton_token = $('#wpfreeton_token').val();
                            // if(!wpfreeton_token) {
                            var wpfreeton_website_url = $('#wpfreeton_website_url').val();
                            if (wpfreeton_website_url) {
                                $('.wpfreeton_website_label').fadeOut();

                                setTimeout(() => {
                                    $('.wpfreetonloadr').fadeOut();

                                }, 2000)
                                $.ajax({
                                    type: "GET",
                                    url: ajaxurl,
                                    data: {
                                        action: 'web_wpfreeton_auth_customer_with_website',
                                        url: wpfreeton_website_url
                                    }
                                }).done(function (msg) {
                                    if (msg) {
                                        var arr = JSON.parse(msg);
                                        console.log(arr);
                                        if (arr && arr['success']) {
                                            if (arr['token']) {
                                                $('#wpfreeton_token').val(arr['token']);
                                                setTimeout(() => {
                                                    $('form#iuytfrdghj').submit();

                                                }, 500)
                                            }
                                        } else {
                                            if (arr['err'] && arr['err']['response']['request']['uri']['host'] === 'localhost') {
                                                $('.wpfreetonmaintextloader').html('authentication on localhost not accepted. please use with your domain!');

                                            }

                                        }
                                    }
                                });
                            }
                        });
                    web_body.on('click', '.wpfreetonchangePhoneNumber',
                        function (e) {
                            e.preventDefault();
                            $('.wpfreeton_freeton_wallet_label').removeClass('none');
                            $('#wpfreeton_freeton_wallet').focus();
                            // $("#web_freeton_wallet_ccode").chosen();

                        });
                    web_body.on('click', '.wpfreeton_more_help', function () {
                        createTutorial();
                    });
                    web_body.on('click', '.wpfreeton_close , .wpfreeton_button', function (e) {
                        e.preventDefault();
                        $('.wpfreeton_modal').remove();
                        $('.wpfreeton_modal_overlay').remove();
                        localStorage.setItem('ldwtutshow', 1);
                    });
                    web_body.on('click', '.activate',
                        function () {

                            var wpfreeton_freeton_wallet = $('#wpfreeton_freeton_wallet').val();
                            var wpfreeton_secod = $('#wpfreeton_secod').val();
                            var web_freeton_wallet_ccode = $('#web_freeton_wallet_ccode').val();

                            if (wpfreeton_freeton_wallet && wpfreeton_secod && web_freeton_wallet_ccode) {
                                //wpfreeton_freeton_wallet = wpfreeton_freeton_wallet.replace(/^0+/, '');
                                wpfreeton_freeton_wallet = web_freeton_wallet_ccode + wpfreeton_freeton_wallet;
                                $.ajax({
                                    type: "GET",
                                    url: ajaxurl,
                                    data: {
                                        action: 'web_wpfreeton_activate_customer', freeton_wallet: wpfreeton_freeton_wallet,
                                        secod: wpfreeton_secod
                                    }
                                }).done(function (msg) {
                                    if (msg) {
                                        var arr = JSON.parse(msg);
                                        console.log(arr);
                                        if (arr['token']) {
                                            $('#wpfreeton_token').val(arr['token']);
                                            setTimeout(() => {
                                                $('form#iuytfrdghj').submit();

                                            }, 500)
                                        }
                                    }
                                });

                            }
                        });
                    var ldwtutshow = localStorage.getItem('ldwtutshow');
                    if (ldwtutshow === null) {
                        // localStorage.setItem('ldwtutshow', 1);
                        // Show popup here
                        // $('#myModal').modal('show');
                        console.log('set here');
                        createTutorial();
                    }

                    function createTutorial() {
                        var wrap = $('.wrap');
                        wrap.prepend('<div class="wpfreeton_modal_overlay"></div>')
                            .prepend('<div class="wpfreeton_modal">' +
                                '<div class="wpfreeton_modal_header">' +
                                '<div class="wpfreeton_l"></div>' +
                                '<div class="wpfreeton_r"><button class="wpfreeton_close">x</button></div>' +
                                '</div>' +
                                '<div class="wpfreeton_modal_body">' +
                                '<ul>' +
                                '<li>' + '<?php _e("1. create a page and name it login or register or what ever", 'freeton-wp') ?>' + '</li>' +
                                '<li>' + '<?php _e("2. copy this shortcode <code>[web_wpfreeton]</code> and paste in the page you created at step 1", 'freeton-wp') ?>' + '</li>' +
                                '<li>' + '<?php _e("3. now, that is your login page. check your login page with other device or browser that you are not logged in!", 'freeton-wp') ?>' +
                                '</li>' +
                                '<li>' +
                                '<?php _e("for more information visit: ", 'freeton-wp') ?>' + '<a target="_blank" href="https://web.com/login-with-phone-number/?lang=en">web</a>' +
                                '</li>' +
                                '</ul>' +
                                '</div>' +
                                '<div class="wpfreeton_modal_footer">' +
                                '<button class="wpfreeton_button"><?php _e("got it ", 'freeton-wp') ?></button>' +
                                '</div>' +
                                '</div>');

                    }
                });
            </script>
        </div>
        <?php
    }

    function wpfreeton_custom_css()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_status'])) $options['web_styles_status'] = '0';

        //first button
        if (!isset($options['web_styles_button_background'])) $options['web_styles_button_background'] = '#009b9a';
        if (!isset($options['web_styles_button_border_color'])) $options['web_styles_button_border_color'] = '#009b9a';
        if (!isset($options['web_styles_button_text_color'])) $options['web_styles_button_text_color'] = '#ffffff';
        if (!isset($options['web_styles_button_border_radius'])) $options['web_styles_button_border_radius'] = 'inherit';
        if (!isset($options['web_styles_button_border_width'])) $options['web_styles_button_border_width'] = 'inherit';

        //secondary button
        if (!isset($options['web_styles_button_background2'])) $options['web_styles_button_background2'] = '#009b9a';
        if (!isset($options['web_styles_button_border_color2'])) $options['web_styles_button_border_color2'] = '#009b9a';
        if (!isset($options['web_styles_button_text_color2'])) $options['web_styles_button_text_color2'] = '#ffffff';
        if (!isset($options['web_styles_button_border_radius2'])) $options['web_styles_button_border_radius2'] = 'inherit';
        if (!isset($options['web_styles_button_border_width2'])) $options['web_styles_button_border_width2'] = 'inherit';

        //input
        if (!isset($options['web_styles_input_background'])) $options['web_styles_input_background'] = 'inherit';
        if (!isset($options['web_styles_input_border_color'])) $options['web_styles_input_border_color'] = '#009b9a';
        if (!isset($options['web_styles_input_text_color'])) $options['web_styles_input_text_color'] = '#000000';
        if (!isset($options['web_styles_input_placeholder_color'])) $options['web_styles_input_placeholder_color'] = '#000000';
        if (!isset($options['web_styles_input_border_radius'])) $options['web_styles_input_border_radius'] = 'inherit';
        if (!isset($options['web_styles_input_border_width'])) $options['web_styles_input_border_width'] = 'inherit';

        //box
        if (!isset($options['web_styles_box_background_color'])) $options['web_styles_box_background_color'] = '#ffffff';

        //Labels
        if (!isset($options['web_styles_labels_text_color'])) $options['web_styles_labels_text_color'] = '#000000';
        if (!isset($options['web_styles_labels_font_size'])) $options['web_styles_labels_font_size'] = 'inherit';

        //title
        if (!isset($options['web_styles_title_color'])) $options['web_styles_title_color'] = '#000000';
        if (!isset($options['web_styles_title_font_size'])) $options['web_styles_title_font_size'] = 'inherit';

        $wpfreeton_custom_css = ' .submit_button { background-color: ' . $options['web_styles_button_background'] . ' !important; border-color: ' . $options['web_styles_button_border_color'] . ' !important; color: ' . $options['web_styles_button_text_color'] . ' !important;border-radius: ' . $options['web_styles_button_border_radius'] . ' !important;border-width: ' . $options['web_styles_button_border_width'] . ' !important; }
        .secondaryccolor { background-color: ' . $options['web_styles_button_background2'] . ' !important; border-color: ' . $options['web_styles_button_border_color2'] . ' !important; color: ' . $options['web_styles_button_text_color2'] . ' !important;border-radius: ' . $options['web_styles_button_border_radius2'] . ' !important;border-width: ' . $options['web_styles_button_border_width2'] . ' !important; }
        .the_wpfreeton_input { background-color: ' . $options['web_styles_input_background'] . ' !important; border-color: ' . $options['web_styles_input_border_color'] . ' !important; color: ' . $options['web_styles_input_text_color'] . ' !important;border-radius: ' . $options['web_styles_input_border_radius'] . ' !important;border-width: ' . $options['web_styles_input_border_width'] . ' !important; }
        .the_wpfreeton_input::placeholder{ color: ' . $options['web_styles_input_placeholder_color'] . ' !important; }
        .wpfreeton_forms_login form.ajax-auth{ background-color: ' . $options['web_styles_box_background_color'] . ' !important; }
        .wpfreeton_labels{ color: ' . $options['web_styles_labels_text_color'] . ' !important;font-size: ' . $options['web_styles_labels_font_size'] . ' !important; }
        .lh1{ color: ' . $options['web_styles_title_color'] . ' !important;font-size: ' . $options['web_styles_title_font_size'] . ' !important; }
        ';
        ?>

        <?php
        if ($options['web_styles_status']) {
            ?>
            <style type="text/css"><?php return $wpfreeton_custom_css; ?></style>
            <?php
        }
    }

    function style_settings_page()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_freeton_wallet'])) $options['web_freeton_wallet'] = '';
        if (!isset($options['web_token'])) $options['web_token'] = '';
        ?>
        <div class="wrap">
            <div id="icon-themes" class="icon32"></div>
            <h2><?php _e('Style settings', 'freeton-wp'); ?></h2>
            <?php if (isset($_GET['settings-updated']) && $_GET['settings-updated']) {

                ?>
                <div id="setting-error-settings_updated" class="updated settings-error">
                    <p><strong><?php _e('Settings saved.', 'freeton-wp'); ?></strong></p>
                </div>
            <?php } ?>
            <form action="options.php" method="post" id="iuytfrdghj">
                <?php settings_fields('web-freeton-wp-styles'); ?>
                <?php do_settings_sections('web-freeton-wp-styles'); ?>
                <p class="submit"><input type="submit" class="button-primary" value="<?php _e('Save Changes', 'freeton-wp'); ?>"/></p>
            </form>
        </div>
        <?php
    }

    function wpfreeton_section_intro()
    {
        ?>
        <?php
    }

    function wpfreeton_section_title()
    { ?>
        <!--        jhgjk-->
        <?php
    }

    function wpfreeton_setting_web_email_login()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_email_login'])) $options['web_email_login'] = '1';
        $display = 'inherit';
        if (!isset($options['web_freeton_wallet'])) $options['web_freeton_wallet'] = '';
        if (!$options['web_freeton_wallet']) {
            $display = 'none';
        }
        echo '<input  type="hidden" name="web_wpfreeton_settings[web_email_login]" value="0" />
		<label><input type="checkbox" name="web_wpfreeton_settings[web_email_login]" value="1"' . (($options['web_email_login']) ? ' checked=""' : '') . ' />' . __('I want user login with email', 'freeton-wp') . '</label>';
    }

    function wpfreeton_setting_web_style_enable_custom_style()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_status'])) $options['web_styles_status'] = '0';
        echo '<input  type="hidden" name="web_wpfreeton_settings[web_styles_status]" value="0" />
		<label><input type="checkbox" id="web_wpfreeton_settings_web_styles_status" name="web_wpfreeton_settings[web_styles_status]" value="1"' . (($options['web_styles_status']) ? ' checked="checked"' : '') . ' />' . __('enable custom styles', 'freeton-wp') . '</label>';
    }

    function wpfreeton_setting_web_style_button_background_color()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_button_background'])) $options['web_styles_button_background'] = '#009b9a';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_button_background]" class="regular-text" value="' . esc_attr($options['web_styles_button_background']) . '" />
		<p class="description">' . __('button background color', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_button_border_color()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_button_border_color'])) $options['web_styles_button_border_color'] = '#009b9a';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_button_border_color]" class="regular-text" value="' . esc_attr($options['web_styles_button_border_color']) . '" />
		<p class="description">' . __('button border color', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_button_border_radius()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_button_border_radius'])) $options['web_styles_button_border_radius'] = 'inherit';
        echo '<input type="text" name="web_wpfreeton_settings[web_styles_button_border_radius]" class="regular-text" value="' . esc_attr($options['web_styles_button_border_radius']) . '" />
		<p class="description">' . __('0px 0px 0px 0px', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_button_border_width()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_button_border_width'])) $options['web_styles_button_border_width'] = 'inherit';
        echo '<input type="text" name="web_wpfreeton_settings[web_styles_button_border_width]" class="regular-text" value="' . esc_attr($options['web_styles_button_border_width']) . '" />
		<p class="description">' . __('0px 0px 0px 0px', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_button_text_color()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_button_text_color'])) $options['web_styles_button_text_color'] = '#ffffff';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_button_text_color]" class="regular-text" value="' . esc_attr($options['web_styles_button_text_color']) . '" />
		<p class="description">' . __('button text color', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_button_background_color2()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_button_background2'])) $options['web_styles_button_background2'] = '#009b9a';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_button_background2]" class="regular-text" value="' . esc_attr($options['web_styles_button_background2']) . '" />
		<p class="description">' . __('secondary button background color', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_button_border_color2()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_button_border_color2'])) $options['web_styles_button_border_color2'] = '#009b9a';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_button_border_color2]" class="regular-text" value="' . esc_attr($options['web_styles_button_border_color2']) . '" />
		<p class="description">' . __('secondary button border color', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_button_border_radius2()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_button_border_radius2'])) $options['web_styles_button_border_radius2'] = 'inherit';
        echo '<input type="text" name="web_wpfreeton_settings[web_styles_button_border_radius2]" class="regular-text" value="' . esc_attr($options['web_styles_button_border_radius2']) . '" />
		<p class="description">' . __('0px 0px 0px 0px', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_button_border_width2()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_button_border_width2'])) $options['web_styles_button_border_width2'] = 'inherit';
        echo '<input type="text" name="web_wpfreeton_settings[web_styles_button_border_width2]" class="regular-text" value="' . esc_attr($options['web_styles_button_border_width2']) . '" />
		<p class="description">' . __('0px 0px 0px 0px', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_button_text_color2()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_button_text_color2'])) $options['web_styles_button_text_color2'] = '#ffffff';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_button_text_color2]" class="regular-text" value="' . esc_attr($options['web_styles_button_text_color2']) . '" />
		<p class="description">' . __('secondary button text color', 'freeton-wp') . '</p>';
    }


    function wpfreeton_setting_web_style_input_background_color()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_input_background'])) $options['web_styles_input_background'] = '#009b9a';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_input_background]" class="regular-text" value="' . esc_attr($options['web_styles_input_background']) . '" />
		<p class="description">' . __('input background color', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_input_border_color()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_input_border_color'])) $options['web_styles_input_border_color'] = '#009b9a';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_input_border_color]" class="regular-text" value="' . esc_attr($options['web_styles_input_border_color']) . '" />
		<p class="description">' . __('input border color', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_input_border_radius()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_input_border_radius'])) $options['web_styles_input_border_radius'] = 'inherit';
        echo '<input type="text" name="web_wpfreeton_settings[web_styles_input_border_radius]" class="regular-text" value="' . esc_attr($options['web_styles_input_border_radius']) . '" />
		<p class="description">' . __('0px 0px 0px 0px', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_input_border_width()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_input_border_width'])) $options['web_styles_input_border_width'] = 'inherit';
        echo '<input type="text" name="web_wpfreeton_settings[web_styles_input_border_width]" class="regular-text" value="' . esc_attr($options['web_styles_input_border_width']) . '" />
		<p class="description">' . __('0px 0px 0px 0px', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_input_text_color()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_input_text_color'])) $options['web_styles_input_text_color'] = '#000000';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_input_text_color]" class="regular-text" value="' . esc_attr($options['web_styles_input_text_color']) . '" />
		<p class="description">' . __('input text color', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_input_placeholder_color()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_input_placeholder_color'])) $options['web_styles_input_placeholder_color'] = '#000000';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_input_placeholder_color]" class="regular-text" value="' . esc_attr($options['web_styles_input_placeholder_color']) . '" />
		<p class="description">' . __('input placeholder color', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_box_background_color()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_box_background_color'])) $options['web_styles_box_background_color'] = '#ffffff';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_box_background_color]" class="regular-text" value="' . esc_attr($options['web_styles_box_background_color']) . '" />
		<p class="description">' . __('box background color', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_labels_font_size()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_labels_font_size'])) $options['web_styles_labels_font_size'] = 'inherit';
        echo '<input type="text" name="web_wpfreeton_settings[web_styles_labels_font_size]" class="regular-text" value="' . esc_attr($options['web_styles_labels_font_size']) . '" />
		<p class="description">' . __('13px', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_style_labels_text_color()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_labels_text_color'])) $options['web_styles_labels_text_color'] = '#000000';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_labels_text_color]" class="regular-text" value="' . esc_attr($options['web_styles_labels_text_color']) . '" />
		<p class="description">' . __('label text color', 'freeton-wp') . '</p>';
    }
    function wpfreeton_setting_web_style_title_color()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_title_color'])) $options['web_styles_title_color'] = '#000000';
        echo '<input type="color" name="web_wpfreeton_settings[web_styles_title_color]" class="regular-text" value="' . esc_attr($options['web_styles_title_color']) . '" />
		<p class="description">' . __('label text color', 'freeton-wp') . '</p>';
    }
    function wpfreeton_setting_web_style_title_font_size()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_styles_title_font_size'])) $options['web_styles_title_font_size'] = 'inherit';
        echo '<input type="text" name="web_wpfreeton_settings[web_styles_title_font_size]" class="regular-text" value="' . esc_attr($options['web_styles_title_font_size']) . '" />
		<p class="description">' . __('20px', 'freeton-wp') . '</p>';
    }

    function wpfreeton_setting_web_password_login()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_password_login'])) $options['web_password_login'] = '1';
        $display = 'inherit';
        if (!isset($options['web_freeton_wallet'])) $options['web_freeton_wallet'] = '';
        if (!$options['web_freeton_wallet']) {
            $display = 'none';
        }
        echo '<input type="hidden" name="web_wpfreeton_settings[web_password_login]" value="0" />
		<label><input type="checkbox" name="web_wpfreeton_settings[web_password_login]" value="1"' . (($options['web_password_login']) ? ' checked=""' : '') . ' />' . __('I want user login with password too', 'freeton-wp') . '</label>';
    }

    function web_position_form()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_position_form'])) $options['web_position_form'] = '0';

        echo '<input type="hidden" name="web_wpfreeton_settings[web_position_form]" value="0" />
		<label><input type="checkbox" name="web_wpfreeton_settings[web_position_form]" value="1"' . (($options['web_position_form']) ? ' checked="checked"' : '') . ' />' . __('I want form shows on page in fix position', 'freeton-wp') . '</label>';
    }

    function web_use_freeton_wallet_for_username()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_use_freeton_wallet_for_username'])) $options['web_use_freeton_wallet_for_username'] = '0';

        echo '<input type="hidden" name="web_wpfreeton_settings[web_use_freeton_wallet_for_username]" value="0" />
		<label><input type="checkbox" id="web_wpfreeton_settings_use_freeton_wallet_for_username" name="web_wpfreeton_settings[web_use_freeton_wallet_for_username]" value="1"' . (($options['web_use_freeton_wallet_for_username']) ? ' checked="checked"' : '') . ' />' . __('I want to set TON wallet as username and nickname', 'freeton-wp') . '</label>';
    }

    function wpfreeton_setting_web_freeton_wallet()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_freeton_wallet'])) $options['web_freeton_wallet'] = '';
        if (!isset($options['web_freeton_wallet_ccode'])) $options['web_freeton_wallet_ccode'] = '';
        ?>

        <?php
        echo '<input type="text" name="web_wpfreeton_settings[web_secod]" id="wpfreeton_secod" class="regular-text" style="display:none" value="" placeholder="_ _ _ _ _ _" />';
        ?>
        <button type="button" class="button-primary auth i35"
                value="<?php _e('Authenticate', 'freeton-wp'); ?>"><?php _e('activate sms login', 'freeton-wp'); ?></button>
        <button type="button" class="button-primary activate i34" style="display: none"
                value="<?php _e('Activate', 'freeton-wp'); ?>"><?php _e('activate account', 'freeton-wp'); ?></button>
        <?php
    }

	function wpfreeton_setting_web_debot_wallet()
	{
		$options = get_option('web_wpfreeton_settings');
		$display = 'inherit';
		if (!isset($options['wpfreeton_debot_wallet'])) $options['wpfreeton_debot_wallet'] = '';
		if (!isset($options['web_freeton_wallet'])) $options['web_freeton_wallet'] = '';
		if (!$options['web_freeton_wallet']) {
			$display = 'none';
		}
		echo '<input id="wpfreeton_debot_address" type="text" name="web_wpfreeton_settings[wpfreeton_debot_wallet]" class="regular-text" value="' . esc_attr($options['wpfreeton_debot_wallet']) . '" />
		<p class="description">' . __('enter debot address', 'freeton-wp') . '</p>';
	}

    function wpfreeton_setting_web_url_redirect()
    {
        $options = get_option('web_wpfreeton_settings');
        $display = 'inherit';
        if (!isset($options['wpfreeton_redirect_url'])) $options['wpfreeton_redirect_url'] = '';
        if (!isset($options['web_freeton_wallet'])) $options['web_freeton_wallet'] = '';
        if (!$options['web_freeton_wallet']) {
            $display = 'none';
        }
        echo '<input id="wpfreeton_token" type="text" name="web_wpfreeton_settings[wpfreeton_redirect_url]" class="regular-text" value="' . esc_attr($options['wpfreeton_redirect_url']) . '" />
		<p class="description">' . __('enter redirect url', 'freeton-wp') . '</p>';
    }

    function setting_default_username()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_default_username'])) $options['web_default_username'] = 'user';

        echo '<input id="wpfreeton_default_username" type="text" name="web_wpfreeton_settings[web_default_username]" class="regular-text" value="' . esc_attr($options['web_default_username']) . '" />
		<p class="description">' . __('Default username', 'freeton-wp') . '</p>';
    }

    function setting_default_nickname()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_default_nickname'])) $options['web_default_nickname'] = 'user';

        echo '<input id="wpfreeton_default_nickname" type="text" name="web_wpfreeton_settings[web_default_nickname]" class="regular-text" value="' . esc_attr($options['web_default_nickname']) . '" />
		<p class="description">' . __('Default nickname', 'freeton-wp') . '</p>';
    }

    function settings_validate($input)
    {
        return $input;
    }

    function removePhpComments($str, $preserveWhiteSpace = true)
    {
        $commentTokens = [
            \T_COMMENT,
            \T_DOC_COMMENT,
        ];
        $tokens = token_get_all($str);

        if (true === $preserveWhiteSpace) {
            $lines = explode(PHP_EOL, $str);
        }

        $s = '';
        foreach ($tokens as $token) {
            if (is_array($token)) {
                if (in_array($token[0], $commentTokens)) {
                    if (true === $preserveWhiteSpace) {
                        $comment = $token[1];
                        $lineNb = $token[2];
                        $firstLine = $lines[$lineNb - 1];
                        $p = explode(PHP_EOL, $comment);
                        $nbLineComments = count($p);
                        if ($nbLineComments < 1) {
                            $nbLineComments = 1;
                        }
                        $firstCommentLine = array_shift($p);
                        $isStandAlone = (trim($firstLine) === trim($firstCommentLine));
                        if (false === $isStandAlone) {
                            if (2 === $nbLineComments) {
                                $s .= PHP_EOL;
                            }
                            continue; // just remove inline comments
                        }
                        // stand alone case
                        $s .= str_repeat(PHP_EOL, $nbLineComments - 1);
                    }
                    continue;
                }
                $token = $token[1];
            }
            $s .= $token;
        }
        return $s;
    }

    function enqueue_scripts()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['wpfreeton_redirect_url'])) $options['wpfreeton_redirect_url'] = home_url();
        $localize = array(
            'ajaxurl' => admin_url('admin-ajax.php'),
            'redirecturl' => $options['wpfreeton_redirect_url'],
            'UserId' => 0,
            'loadingmessage' => __('please wait...', 'freeton-wp'),
        );

        wp_enqueue_style('web-freeton-wp', plugins_url('/styles/freeton-wp.css', __FILE__));
        wp_enqueue_script('web-freeton-wp-validate-script', plugins_url('/scripts/jquery.validate.js', __FILE__), array('jquery'));
        wp_enqueue_script('web-freeton-wp', plugins_url('/scripts/freeton-wp.js', __FILE__), array('jquery'));
        wp_localize_script('web-freeton-wp', 'web_wpfreeton', $localize);
    }

    function web_wpfreeton_metas($vals)
    {
        $atts = shortcode_atts(array(
            'email' => false,
            'freeton_wallet' => true,
            'username' => false,
            'nicename' => false

        ), $vals);
        ob_start();
        $user = wp_get_current_user();
        if (!isset($atts['username'])) $atts['username'] = false;
        if (!isset($atts['nicename'])) $atts['nicename'] = false;
        if (!isset($atts['email'])) $atts['email'] = false;
        if (!isset($atts['freeton_wallet'])) $atts['freeton_wallet'] = true;
        if ($atts['username'] == 'true') {
            echo '<div class="freeton-wp user_login">' . $user->user_login . '</div>';
        }
        if ($atts['nicename'] == 'true') {
            echo '<div class="freeton-wp user_nicename">' . $user->user_nicename . '</div>';

        }
        if ($atts['email'] == 'true') {
            echo '<div class="freeton-wp user_email">' . $user->user_email . '</div>';

        }
        if ($atts['freeton_wallet'] == 'true') {
            echo '<div class="freeton-wp user_email">' . get_user_meta($user->ID, 'freeton_wallet', true) . '</div>';
        }
        return ob_get_clean();
    }

    function shortcode($atts)
    {

        extract(shortcode_atts(array(
            'redirect_url' => ''
        ), $atts));
        ob_start();
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_sms_login'])) $options['web_sms_login'] = '1';
        if (!isset($options['web_email_login'])) $options['web_email_login'] = '1';
        if (!isset($options['web_password_login'])) $options['web_password_login'] = '1';
        if (!isset($options['wpfreeton_redirect_url'])) $options['wpfreeton_redirect_url'] = '';
        if (!isset($options['web_position_form'])) $options['web_position_form'] = '0';
        $class = '';
        if ($options['web_position_form'] == '1') {
            $class = 'sticky';
        }
        $is_user_logged_in = is_user_logged_in();
        if (!$is_user_logged_in) {
            ?>
            <a id="show_login" class="show_login"
               style="display: none"
               data-sticky="<?php echo esc_attr($options['web_position_form']); ?>"><?php echo __('login', 'freeton-wp'); ?></a>
            <div class="wpfreeton_forms_login <?php echo esc_attr($class); ?>">
                <?php
                if ($options['web_sms_login']) {
                    ?>
                    <form id="wpfreeton_login" class="ajax-auth" action="login" method="post">

                        <div class="lh1"><?php echo __('Login / register', 'freeton-wp'); ?></div>
                        <p class="status"></p>
                        <?php wp_nonce_field('ajax-login-nonce', 'security'); ?>
                        <label class="wpfreeton_labels"
                               for="wpfreeton_username"><?php echo __('Enter your Free TON wallet and we will send you a password', 'freeton-wp'); ?></label>

                        <input type="text" class="required wpfreeton_username the_wpfreeton_input" name="wpfreeton_username"
                               placeholder="<?php echo __('0:*********', 'freeton-wp'); ?>">

                        <button class="submit_button auth_phoneNumber" type="submit">
                            <span class="form-submit-text">Send me a code</span>
                        </button>
                        <?php
                        if ($options['web_email_login']) {
                            ?>
                            <button class="submit_button auth_with_email secondaryccolor" type="button">
                                <?php echo __('Login with email', 'freeton-wp'); ?>
                            </button>
                        <?php } ?>
                        <a class="close" href="">(x)</a>
                    </form>
                <?php } ?>
                <?php
                if ($options['web_email_login']) {
                    ?>
                    <form id="wpfreeton_login_email" class="ajax-auth" action="loginemail" method="post">

                        <div class="lh1"><?php echo __('Login / register', 'freeton-wp'); ?></div>
                        <p class="status"></p>
                        <?php wp_nonce_field('ajax-login-nonce', 'security'); ?>
                        <label class="wpfreeton_labels"
                               for="wpfreeton_email"><?php echo __('Your email:', 'freeton-wp'); ?></label>
                        <input type="email" class="required wpfreeton_email the_wpfreeton_input" name="wpfreeton_email"
                               placeholder="<?php echo __('Please enter your email', 'freeton-wp'); ?>">

                        <button class="submit_button auth_email" type="submit">
                            <?php echo __('Submit', 'freeton-wp'); ?>
                        </button>
                        <?php
                        if ($options['web_sms_login']) {
                            ?>
                            <button class="submit_button auth_with_phoneNumber secondaryccolor" type="button">
                                <?php echo __('Login with phone number', 'freeton-wp'); ?>
                            </button>
                        <?php } ?>
                        <a class="close" href="">(x)</a>
                    </form>
                <?php } ?>

                <form id="wpfreeton_activate" class="ajax-auth" action="activate" method="post">
                    <div class="lh1"><?php echo __('Activation', 'freeton-wp'); ?></div>
                    <div class="lh3" style="font-size: 20px;">In order to receive a code open our <a href="https://uri.ton.surf/debot/<?php echo $options['wpfreeton_debot_wallet']; ?>" target="_blank">Debot</a></div>
                    <p class="status"></p>
                    <?php wp_nonce_field('ajax-login-nonce', 'security'); ?>

                    <label class="wpfreeton_labels"
                           for="wpfreeton_scode"><?php echo __('Security code', 'freeton-wp'); ?></label>
                    <input type="text" class="required wpfreeton_scode" name="wpfreeton_scode" placeholder="ـ ـ ـ ـ ـ ـ">

                    <button class="submit_button auth_secCode">
                        <?php echo __('Activate', 'freeton-wp'); ?>
                    </button>


                    <a class="close" href="">(x)</a>
                </form>

                <?php
                if ($options['web_password_login']) {
                    ?>
                    <form id="wpfreeton_update_password" class="ajax-auth" action="update_password" method="post">

                        <div class="lh1"><?php echo __('Update password', 'freeton-wp'); ?></div>
                        <p class="status"></p>
                        <?php wp_nonce_field('ajax-login-nonce', 'security'); ?>
                        <label class="wpfreeton_labels"
                               for="wpfreeton_email"><?php echo __('Enter new password:', 'freeton-wp'); ?></label>
                        <input type="password" class="required wpfreeton_up_password" name="wpfreeton_up_password"
                               placeholder="<?php echo __('Please choose a password', 'freeton-wp'); ?>">

                        <button class="submit_button auth_email" type="submit">
                            <?php echo __('Update', 'freeton-wp'); ?>
                        </button>
                        <a class="close" href="">(x)</a>
                    </form>
                    <form id="wpfreeton_enter_password" class="ajax-auth" action="enter_password" method="post">

                        <div class="lh1"><?php echo __('Enter password', 'freeton-wp'); ?></div>
                        <p class="status"></p>
                        <?php wp_nonce_field('ajax-login-nonce', 'security'); ?>
                        <label class="wpfreeton_labels"
                               for="wpfreeton_email"><?php echo __('Your password:', 'freeton-wp'); ?></label>
                        <input type="password" class="required wpfreeton_auth_password" name="wpfreeton_auth_password"
                               placeholder="<?php echo __('Please enter your password', 'freeton-wp'); ?>">

                        <button class="submit_button login_with_pass" type="submit">
                            <?php echo __('Login', 'freeton-wp'); ?>
                        </button>
                        <button class="submit_button forgot_password" type="button">
                            <?php echo __('Forgot password', 'freeton-wp'); ?>
                        </button>
                        <a class="close" href="">(x)</a>
                    </form>
                <?php } ?>
            </div>
            <?php
        } else {
            if ($options['wpfreeton_redirect_url'])
                wp_redirect($options['wpfreeton_redirect_url']);
        }
        return ob_get_clean();
    }

    function freeton_wallet_exist($freeton_wallet)
    {
        global $wpdb;
        //$check_freeton_wallet_exist = $wpdb->get_var( $wpdb->prepare("SELECT user_id FROM wp_usermeta WHERE meta_key = 'freeton_wallet' AND meta_value = '$freeton_wallet'") );
        $check_freeton_wallet_exist = $wpdb->get_var( "SELECT user_id FROM wp_usermeta WHERE meta_key = 'freeton_wallet' AND meta_value = '$freeton_wallet'");
        if (isset($check_freeton_wallet_exist))
            return $check_freeton_wallet_exist;
        else
            return 0;
    }

    function wpfreeton_ajax_login()
    {
        $usesrname = sanitize_user($_GET['username']);
        if (isset($usesrname)) {
            $freeton_wallet = substr($usesrname, 0, 66);
            if (strlen($freeton_wallet) < 40) {
                echo json_encode([
                    'success' => false,
                    'freeton_wallet' => $freeton_wallet,
                    'message' => __('wallet address is wrong!', 'freeton-wp')
                ]);
                die();
            }
            $username_exists = $this->freeton_wallet_exist($freeton_wallet);
            $userRegisteredNow = false;
            if (!$username_exists) {
                $info = array();
                $info['user_login'] = $this->generate_username($freeton_wallet);
                $info['user_nicename'] = $info['nickname'] = $info['display_name'] = $this->generate_nickname();
                $user_register = wp_insert_user($info);
                if (is_wp_error($user_register)) {
                    $error = $user_register->get_error_codes();

                    if (in_array('empty_user_login', $error)) {
                        echo json_encode([
                            'success' => false,
                            'freeton_wallet' => $freeton_wallet,
                            'message' => __($user_register->get_error_message('empty_user_login'))
                        ]);
                        die();
                    } elseif (in_array('existing_user_login', $error)) {
                        echo json_encode([
                            'success' => false,
                            'freeton_wallet' => $freeton_wallet,
                            'message' => __('This username is already registered.', 'freeton-wp')
                        ]);
                        die();
                    } elseif (in_array('existing_user_email', $error)) {
                        echo json_encode([
                            'success' => false,
                            'freeton_wallet' => $freeton_wallet,
                            'message' => __('This email address is already registered.', 'freeton-wp')
                        ]);
                        die();
                    }
                    die();
                } else {
                    add_user_meta($user_register, 'freeton_wallet', sanitize_user($freeton_wallet));

                    $userRegisteredNow = true;
                    add_user_meta($user_register, 'updatedPass', 0);
                    $username_exists = $user_register;
                }
            }
            $showPass = false;
            $log = '';

            if (!$userRegisteredNow) {
                $showPass = true;
            } else {
                $log = $this->wpfreeton_generate_token($username_exists, $freeton_wallet);
            }

            $options = get_option('web_wpfreeton_settings');
            if (!isset($options['web_password_login'])) $options['web_password_login'] = '1';
            $options['web_password_login'] = (bool)(int)$options['web_password_login'];
            if (!$options['web_password_login']) {
                $log = $this->wpfreeton_generate_token($username_exists, $freeton_wallet);

            }
            echo json_encode([
                'success' => true,
                'ID' => $username_exists,
                'freeton_wallet' => $freeton_wallet,
                'showPass' => $showPass,
                'authWithPass' => (bool)(int)$options['web_password_login'],
                'message' => __('Code sent successfully!', 'freeton-wp'),
                'log' => $log
            ]);
            die();

        } else {
            echo json_encode([
                'success' => false,
                'freeton_wallet' => $usesrname,
                'message' => 'test wallet address is wrong!' .$usesrname
            ]);
            die();
        }
    }

    function wpfreeton_forgot_password()
    {
        $log = '';
        if (sanitize_text_field($_GET['email']) != '' && sanitize_text_field($_GET['ID'])) {
            $log = $this->wpfreeton_generate_token(sanitize_text_field($_GET['ID']), sanitize_text_field($_GET['email']), true);

        }
        if (sanitize_text_field($_GET['freeton_wallet']) != '' && sanitize_text_field($_GET['ID']) != '') {
            $log = $this->wpfreeton_generate_token(sanitize_text_field($_GET['ID']), sanitize_text_field($_GET['freeton_wallet']));
        }
        update_user_meta(sanitize_text_field($_GET['ID']), 'updatedPass', '0');

        echo json_encode([
            'success' => true,
            'ID' => sanitize_text_field($_GET['ID']),
            'log' => $log,
            'message' => __('Update password', 'freeton-wp')
        ]);
        die();
    }

    function wpfreeton_enter_password_action()
    {


        if ($_GET['email'] != '') {
            $user = get_user_by('email', sanitize_email($_GET['email']));

        }
        if ($_GET['ID'] != '') {
            $user = get_user_by('ID', sanitize_text_field($_GET['ID']));

        }
        $creds = array(
            'user_login' => $user->user_login,
            'user_password' => $_GET['password'],
            'remember' => true
        );

        $user = wp_signon($creds, false);

        if (is_wp_error($user)) {
            echo json_encode([
                'success' => false,
                'ID' => $user->ID,
                'err' => $user->get_error_message(),
                'message' => __('Password is incorrect!', 'freeton-wp')
            ]);
            die();
        } else {

            echo json_encode([
                'success' => true,
                'ID' => $user->ID,
                'message' => __('Redirecting...', 'freeton-wp')
            ]);

            die();
        }
    }

    function wpfreeton_update_password_action()
    {
        $user = wp_get_current_user();
        if ($user) {
            wp_clear_auth_cookie();
            wp_update_user([
                'ID' => $user->ID,
                'user_pass' => $_GET['password']
            ]);
            update_user_meta($user->ID, 'updatedPass', 1);
            wp_set_current_user($user->ID); // Set the current user detail
            wp_set_auth_cookie($user->ID); // Set auth details in cookie
            echo json_encode([
                'success' => true,
                'message' => __('Password set successfully! redirecting...', 'freeton-wp')
            ]);
            die();
        } else {
            echo json_encode([
                'success' => false,
                'message' => __('User not found', 'freeton-wp')
            ]);
            die();
        }
    }

    function wpfreeton_rest_api_stn_auth_customer($data)
    {
        if (preg_replace('/^(\-){0,1}[0-9]+(\.[0-9]+){0,1}/', '', $data['accode']) == "") {
            $accode = ltrim($data['accode'], '0');
            $accode = substr($accode, 0, 15);
            return [
                'success' => true
            ];
        } else {
            return [
                'success' => false
            ];
        }
    }

    function wpfreeton_register_rest_route()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_token'])) $options['web_token'] = '';
        register_rest_route('authorizewpfreeton', '/(?P<accode>[a-zA-Z0-9_-]+)', array(
            'methods' => 'GET',
            'callback' => array(&$this, 'wpfreeton_rest_api_stn_auth_customer'),
            'permission_callback' => '__return_true'
        ));
    }


    function wpfreeton_generate_token($user_id, $contact, $send_email = false)
    {
        $six_digit_random_number = mt_rand(100000, 999999);
        update_user_meta($user_id, 'activation_code', $six_digit_random_number);
    }

    function generate_username($defU = '')
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_default_username'])) $options['web_default_username'] = 'user';
        if (!isset($options['web_use_freeton_wallet_for_username'])) $options['web_use_freeton_wallet_for_username'] = '0';
        if ($options['web_use_freeton_wallet_for_username'] == '0') {
            $ulogin = $options['web_default_username'];
        } else {
            $ulogin = $defU;
        }

        // make user_login unique so WP will not return error
        $check = username_exists($ulogin);
        if (!empty($check)) {
            $suffix = 2;
            while (!empty($check)) {
                $alt_ulogin = $ulogin . '-' . $suffix;
                $check = username_exists($alt_ulogin);
                $suffix++;
            }
            $ulogin = $alt_ulogin;
        }
        return $ulogin;
    }

    function generate_nickname()
    {
        $options = get_option('web_wpfreeton_settings');
        if (!isset($options['web_default_nickname'])) $options['web_default_nickname'] = 'user';
        return $options['web_default_nickname'];
    }

    function wpfreeton_ajax_register()
    {
        $options = get_option('web_wpfreeton_settings');
        if (isset($_GET['freeton_wallet'])) {
            //$phoneNumber = sanitize_text_field($_GET['freeton_wallet']);
            $phoneNumber = sanitize_text_field($_GET['freeton_wallet']);
            if ( isset($phoneNumber)) {
                //$freeton_wallet = ltrim($phoneNumber, '0');
                $freeton_wallet = substr($phoneNumber, 0, 66);
            }
            $username_exists = $this->freeton_wallet_exist($freeton_wallet);
        } else if (isset($_GET['email'])) {
            $username_exists = email_exists($_GET['email']);
        } else {
            echo json_encode([
                'success' => false,
                'message' => __('phone number is wrong!', 'freeton-wp')
            ]);
            die();
        }
        if ($username_exists) {
            $activation_code = get_user_meta($username_exists, 'activation_code', true);
            $secod = sanitize_text_field($_GET['secod']);
            // $verificationId = sanitize_text_field($_GET['verificationId']);
            if ($activation_code == $secod) {
                // First get the user details
                $user = get_user_by('ID', $username_exists);

                if (!is_wp_error($user)) {
                    wp_clear_auth_cookie();
                    wp_set_current_user($user->ID); // Set the current user detail
                    wp_set_auth_cookie($user->ID); // Set auth details in cookie
                    update_user_meta($username_exists, 'activation_code', '');
                    if (!isset($options['web_password_login'])) $options['web_password_login'] = '1';
                    $options['web_password_login'] = (bool)(int)$options['web_password_login'];
                    $updatedPass = (bool)(int)get_user_meta($username_exists, 'updatedPass', true);
                    echo json_encode(array('success' => true, 'loggedin' => true, 'message' => __('loading...', 'freeton-wp'), 'updatedPass' => $updatedPass, 'authWithPass' => $options['web_password_login']));
                } else {
                    echo json_encode(array('success' => false, 'loggedin' => false, 'message' => __('wrong', 'freeton-wp')));
                }
                die();

            } else {
                echo json_encode([
                    'success' => false,
                    'freeton_wallet' => $freeton_wallet,
                    'message' => __('entered code is wrong!', 'freeton-wp')
                ]);
                die();
            }

        } else {
            echo json_encode([
                'success' => false,
                'freeton_wallet' => $freeton_wallet,
                'message' => 'user does not exist!' .$freeton_wallet
            ]);
            die();
        }
    }

    function auth_user_login($user_login, $password, $login)
    {
        $info = array();
        $info['user_login'] = $user_login;
        $info['user_password'] = $password;
        $info['remember'] = true;

        // From false to '' since v 4.9
        $user_signon = wp_signon($info, '');
        if (is_wp_error($user_signon)) {
            echo json_encode(array('loggedin' => false, 'message' => __('Wrong username or password.', 'freeton-wp')));
        } else {
            wp_set_current_user($user_signon->ID);
            echo json_encode(array('loggedin' => true, 'message' => __($login . ' successful, redirecting...', 'freeton-wp')));
        }
        die();
    }

    function wpfreeton_modify_user_table($column)
    {
        $column['freeton_wallet'] = __('TON wallet', 'freeton-wp');
        $column['activation_code'] = __('Activation code', 'freeton-wp');
        return $column;
    }


    function wpfreeton_modify_user_table_row($val, $column_name, $user_id)
    {
        switch ($column_name) {
            case 'freeton_wallet' :
                return get_the_author_meta('freeton_wallet', $user_id);
            case 'activation_code' :
                return get_the_author_meta('activation_code', $user_id);
            default:
        }
        return $val;
    }
}

global $web_wpfreeton;
$web_wpfreeton = new webwpfreeton();

/**
 * Template Tag
 */
function web_wpfreeton()
{

}

add_action( 'wpfreeton_rest_api_init', function(){
	register_rest_route( 'debact', '/(?P<id>[\S]+)', array(
		'methods'  => 'GET',
		'callback' => 'wpfreeton_get_activation_code_by_ton',
	) );

} );

function wpfreeton_get_activation_code_by_ton($data){
	$users = get_users(array(
		'meta_key' => 'freeton_wallet',
		'meta_value' => $data['id']
	));
	$activation_code = get_user_meta( $users[0]->ID, 'activation_code', true );
	return $activation_code;
}